<?php

namespace Drupal\text_snip_formatter\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Annotation\Translation;

/**
 * Plugin implementation of the 'text_snip_formatter' formatter.
 *
 * @FieldFormatter(
 *   id = "text_snip_formatter",
 *   label = @Translation("Text Snip Formatter"),
 *   field_types = {
 *     "string",
 *     "text",
 *     "text_long"
 *   }
 * )
 */
class TextSnipFormatter extends FormatterBase {

    /**
     * {@inheritdoc}
     */
    public static function defaultSettings() {
        return [
                'char_count' => 4,
                'position' => 'end',
                'middle_start' => 0, // Default middle start position
            ] + parent::defaultSettings();
    }

    /**
     * {@inheritdoc}
     */
    public function settingsForm(array $form, FormStateInterface $form_state) {
        $elements['char_count'] = [
            '#type' => 'number',
            '#title' => $this->t('Number of characters to display'),
            '#default_value' => $this->getSetting('char_count'),
            '#min' => 1,
            '#description' => $this->t('Specify how many characters of the text should be displayed.'),
        ];

        $elements['position'] = [
            '#type' => 'select',
            '#title' => $this->t('Position to display characters'),
            '#default_value' => $this->getSetting('position'),
            '#options' => [
                'start' => $this->t('Start (beginning of the string)'),
                'middle' => $this->t('Middle (custom start point)'),
                'end' => $this->t('End (last part of the string)'),
            ],
            '#description' => $this->t('Choose where the characters will be displayed from the string.'),
            '#attributes' => [
                'id' => 'edit-position',
            ],
        ];

        // Middle Start Position - Only visible when "Middle" is selected
        $elements['middle_start'] = [
            '#type' => 'number',
            '#title' => $this->t('Middle start position'),
            '#default_value' => $this->getSetting('middle_start'),
            '#min' => 0,
            '#description' => $this->t('Specify the character position from which the middle text should start (0-based index).'),
            '#states' => [
                'visible' => [
                    ':input[id="edit-position"]' => ['value' => 'middle'],
                ],
            ],
        ];

        return $elements;
    }

    /**
     * {@inheritdoc}
     */
    public function settingsSummary() {
        $summary = [];
        $summary[] = $this->t('Displays @count characters from the @position of the string.', [
            '@count' => $this->getSetting('char_count'),
            '@position' => $this->getSetting('position'),
        ]);
        return $summary;
    }

    /**
     * {@inheritdoc}
     */
    public function viewElements(FieldItemListInterface $items, $langcode) {
        $elements = [];
        $char_count = $this->getSetting('char_count');
        $position = $this->getSetting('position');
        $middle_start = $this->getSetting('middle_start');

        foreach ($items as $delta => $item) {
            $value = $item->value;
            $length = strlen($value);

            if ($length <= $char_count) {
                // If the text is shorter than the visible character count, show it all
                $elements[$delta] = ['#markup' => $value];
                continue;
            }

            switch ($position) {
                case 'start':
                    $visible = substr($value, 0, $char_count);
                    $masked = str_repeat('*', $length - $char_count);
                    $elements[$delta] = ['#markup' => $visible . $masked];
                    break;

                case 'middle':
                    // Ensure middle_start is within valid range
                    $middle_start = max(0, min($middle_start, $length - $char_count));
                    $visible = substr($value, $middle_start, $char_count);
                    $masked_before = str_repeat('*', $middle_start);
                    $masked_after = str_repeat('*', $length - ($middle_start + $char_count));
                    $elements[$delta] = ['#markup' => $masked_before . $visible . $masked_after];
                    break;

                case 'end':
                default:
                    $visible = substr($value, -$char_count);
                    $masked = str_repeat('*', $length - $char_count);
                    $elements[$delta] = ['#markup' => $masked . $visible];
                    break;
            }
        }

        return $elements;
    }
}
